@@ -1,7 +1,7 @@
 #ifndef __MAILSYNC_MSGID__
 
 #include <string>
-#include <c-client.h>
+#include "c-client-header.h"
 #include "options.h"
 
 using namespace std;
@@ -22,6 +22,7 @@ class MsgId : public string
     void sanitize_message_id();
     string to_msinfo_format();
     string from_msinfo_format();
+    bool empty();
 };
 
 #define __MAILSYNC_MSGID__
